import React, { useMemo } from "react";
import Fuse from "fuse.js";
import flatMap from "lodash/flatMap";
import { Col } from "react-grid-system";
import { ErrorComponent } from "@gooddata/react-components";

import AttributeList from "./AttributeList";
import MeasureList from "./MeasureList";
import DateDataSetList from "./DateDataSetList";
import AttributeDisplayFormList from "./AttributeDisplayFormList";

import styles from "./CatalogueBrowserResults.module.scss";
import { getAllAttributes, getAllMeasures, getAllDateDataSets } from "../../utils/catalogue";

const createFuse = (items = []) =>
    new Fuse(items, {
        keys: [
            "meta.title",
            "content.displayForms.meta.title",
            "content.availableDateAttributes.attributeMeta.title",
        ],
        // make the search less fuzzy
        threshold: 0.2,
    });

const useFuseSearch = (data, enabled, query) => {
    const fuse = useMemo(() => createFuse(data), [data]);
    if (!enabled) {
        return [];
    }
    return query ? fuse.search(query) : data;
};

const columnLayouts = {
    1: {},
    2: { md: 6 },
    3: { xl: 4 },
    4: { md: 6 },
    5: { xl: 4 },
    6: {},
};

const getColumnLayoutProps = blockCount => columnLayouts[blockCount] || columnLayouts[6];

const CatalogueBrowserResults = ({ data, query, selectedTypes }) => {
    const filteredAttributes = useFuseSearch(
        Object.values(getAllAttributes(data)),
        selectedTypes.attributes,
        query,
    );

    const displayForms = useMemo(
        () => flatMap(Object.values(getAllAttributes(data)), attribute => attribute.content.displayForms),
        [data],
    );
    const filteredAttributeDisplayForms = useFuseSearch(
        displayForms,
        selectedTypes.attributeDisplayForms,
        query,
    );

    const filteredMeasures = useFuseSearch(
        Object.values(getAllMeasures(data)),
        selectedTypes.measures,
        query,
    );

    const filteredDateDataSets = useFuseSearch(
        Object.values(getAllDateDataSets(data)),
        selectedTypes.dateDataSets,
        query,
    );

    const displayedBlocks = [
        { items: filteredAttributeDisplayForms, Component: AttributeDisplayFormList },
        { items: filteredAttributes, Component: AttributeList },
        { items: filteredMeasures, Component: MeasureList },
        { items: filteredDateDataSets, Component: DateDataSetList },
    ].filter(({ items }) => items.length > 0);

    const columnLayout = getColumnLayoutProps(displayedBlocks.length);

    return displayedBlocks.length ? (
        <>
            {displayedBlocks.map(({ items, Component }, blockIndex) => (
                <Col key={blockIndex} {...columnLayout}>
                    <Component
                        items={items}
                        key={blockIndex}
                        className={styles.ResultBlock}
                        projectId={data.projectId}
                    />
                </Col>
            ))}
        </>
    ) : (
        <Col>
            <ErrorComponent icon="icon-ghost" message="No results" />
        </Col>
    );
};

export default CatalogueBrowserResults;
