import React from "react";
import { Container, Row, Col } from "react-grid-system";
import isEmpty from "lodash/isEmpty";
import qs from "qs";
import { withRouter } from "react-router-dom";

import CustomLoading from "../CustomLoading";
import ToggleCheckbox from "../controls/ToggleCheckbox";
import CatalogueBrowserResults from "./CatalogueBrowserResults";

import styles from "./CatalogueBrowser.module.scss";
import sharedStyles from "../../shared.module.scss";
import Input from "../controls/Input";
import useCatalogue from "../../hooks/useCatalogue";

const labelTypeMap = {
    attributes: "Attributes",
    attributeDisplayForms: "Attribute Display Forms",
    measures: "Measures",
    dateDataSets: "Date Data Sets",
};

const defaultSelectedTypes = {
    attributes: false,
    attributeDisplayForms: true,
    measures: true,
    dateDataSets: false,
};
const availableItemTypes = Object.keys(defaultSelectedTypes);
const normalizeBooleans = object =>
    Object.entries(object).reduce((result, [key, value]) => {
        result[key] = value === "true";
        return result;
    }, {});

const CatalogueBrowser = ({ history, location }) => {
    const state = qs.parse(location.search, {
        ignoreQueryPrefix: true,
    });
    const { q, ...selectedTypes } = state;
    const selectedTypesMap = !isEmpty(selectedTypes)
        ? normalizeBooleans(selectedTypes)
        : defaultSelectedTypes;

    const setQueryString = value =>
        history.replace({
            ...location,
            search: qs.stringify(value),
        });

    const updateQuery = q => setQueryString({ ...state, q });

    const updateSelectedType = (type, value) =>
        setQueryString({
            ...state,
            ...selectedTypesMap,
            [type]: value,
        });

    const { data, isLoading } = useCatalogue();

    return (
        <div>
            <h2 className={sharedStyles.Headline}>Catalogue browser</h2>
            <Input
                type="search"
                value={q || ""}
                onChange={e => updateQuery(e.target.value)}
                placeholder="Search&hellip;"
                className={styles.Input}
            />
            <div className={styles.SearchControls}>
                {availableItemTypes.map(type => {
                    const value = selectedTypesMap[type];
                    return (
                        <ToggleCheckbox
                            key={type}
                            value={!!value}
                            className={styles.ToggleCheckbox}
                            onChange={value => updateSelectedType(type, value)}
                        >
                            {labelTypeMap[type] || type}
                        </ToggleCheckbox>
                    );
                })}
            </div>

            <div className={styles.Results}>
                <Container fluid>
                    <Row>
                        {isLoading ? (
                            <Col key="loading">
                                <CustomLoading
                                    height={200}
                                    width={"100%"}
                                    className={styles.Loading}
                                    label="Loading Catalogue Items&hellip;"
                                />
                            </Col>
                        ) : (
                            <CatalogueBrowserResults data={data} query={q} selectedTypes={selectedTypesMap} />
                        )}
                    </Row>
                </Container>
            </div>
        </div>
    );
};

export default withRouter(CatalogueBrowser);
