import React from "react";

import ResultList from "./ResultList";
import ResultListItem from "./ResultListItem";
import ElementList from "./ElementList";
import CodeBlock from "../codeListing/CodeBlock";

import styles from "./DateDataSetList.module.scss";
import ResultSubList from "./ResultSubList";

const DateDataSetHelp = () => {
    return (
        <>
            <p>
                <code>Date Data Sets</code> are metadata objects used to work with date. Different{" "}
                <code>Date Data Sets</code> are the most general date objects that define what type of date we
                want to work with. For example calendar date or fiscal date.
            </p>
            <p>
                <code>Date Data Sets</code> can be used to define:
            </p>
            <ul>
                <li>
                    <p>Filtering visualizations using absolute and relative date filters</p>
                    <CodeBlock>
                        {`import { ColumnChart, Model } from "@gooddata/react-components";

// ...

<ColumnChart
    ...
    filters={[Model.absoluteDateFilter(dateDataSetIdentifier, '2019-01-01', '2019-12-31')]}
/>`}
                    </CodeBlock>
                </li>
                <li>
                    <p>Previous period comparison</p>
                    <CodeBlock>
                        {`import { ColumnChart, Model } from "@gooddata/react-components";

// ...

<ColumnChart
    ...
    measures={[Model.previousPeriodMeasure(measureIdentifier, [{
        dataSet: dateAttributeIdentifier,
        periodsAgo: 1
    }])]}
    filters={[Model.absoluteDateFilter(dateDataSetIdentifier, '2019-01-01', '2019-12-31')]}
/>`}
                    </CodeBlock>
                </li>
            </ul>
            <p>
                <code>Date Data Sets</code> contain <code>Date Attributes</code>
            </p>
            <p>
                <code>Date Attributes</code> can be used to define:
            </p>
            <ul>
                <li>
                    <p>Period over Period comparison</p>
                    <CodeBlock>
                        {`import { ColumnChart, Model } from "@gooddata/react-components";

// ...

<ColumnChart
    ...
    measures={[Model.popMeasure(measureIdentifier, dateAttributeIdentifier)]}
    filters={[Model.absoluteDateFilter(dateDataSetIdentifier, '2019-01-01', '2019-12-31')]}
/>`}
                    </CodeBlock>
                </li>
            </ul>
            <p>
                <code>Date Attributes</code> contain <code>Date Attribute Display Forms</code>
            </p>
            <p>
                Most common uses for <code>Date Attribute Display Forms</code> are:
            </p>
            <ul>
                <li>
                    <p>
                        Props defining Attributes in visualizations e.g. <code>viewBy</code> in a{" "}
                        <code>ColumnChart</code> component.
                    </p>
                    <CodeBlock>
                        {`import { ColumnChart, Model } from "@gooddata/react-components";

// ...

<ColumnChart
    projectId={projectId}
    measures={[Model.measure(measureIdentifier)]}
    viewBy={Model.attribute(dateAttributeDisplayFormIdentifier)}
/>`}
                    </CodeBlock>
                </li>
                <li>
                    <p>Filtering visualizations using Attribute filters:</p>
                    <CodeBlock>
                        {`import { ColumnChart, Model } from "@gooddata/react-components";

// ...

<ColumnChart
    ...
    filters={[Model.attributeFilter(dateAttributeDisplayFormIdentifier)
        .in("2018", "2019")]}
/>`}
                    </CodeBlock>
                </li>
                <li>
                    <p>Sorting by Attribute Display Form value:</p>
                    <CodeBlock>
                        {`import { ColumnChart, Model } from "@gooddata/react-components";

// ...

<ColumnChart
    ...
    sortBy={[Model.attributeSortItem(dateAttributeDisplayFormIdentifier, 'asc')]}
/>`}
                    </CodeBlock>
                </li>
            </ul>
        </>
    );
};

const DateDataSetListItem = ({ dateDataSet, projectId }) => {
    return (
        <ResultListItem
            key={dateDataSet.meta.identifier}
            // className={styles.Item}
            title={dateDataSet.meta.title}
            identifier={dateDataSet.meta.identifier}
        >
            <ResultSubList heading="Date attributes">
                {dateDataSet.availableDateAttributes.map(dateAttribute => (
                    <ResultListItem
                        key={dateAttribute.attributeMeta.identifier}
                        className={styles.DateAttributeItem}
                        title={dateAttribute.attributeMeta.title}
                        identifier={dateAttribute.attributeMeta.identifier}
                    >
                        <ResultSubList heading="Display Forms">
                            {dateAttribute.displayForms.map(attributeDisplayForm => (
                                <ResultListItem
                                    key={attributeDisplayForm.meta.identifier}
                                    title={attributeDisplayForm.meta.title}
                                    identifier={attributeDisplayForm.meta.identifier}
                                >
                                    <ElementList
                                        displayForm={attributeDisplayForm}
                                        projectId={projectId}
                                        className={styles.Elements}
                                    />
                                </ResultListItem>
                            ))}
                        </ResultSubList>
                    </ResultListItem>
                ))}
            </ResultSubList>
        </ResultListItem>
    );
};

const DateDataSetList = ({ items, className, projectId }) => {
    return (
        <ResultList title="Date Data Sets" className={className} headerContent={<DateDataSetHelp />}>
            {items.map(dateDataSet => (
                <DateDataSetListItem
                    key={dateDataSet.meta.identifier}
                    dateDataSet={dateDataSet}
                    projectId={projectId}
                />
            ))}
        </ResultList>
    );
};

export default DateDataSetList;
