import React, { useState, useMemo, useEffect } from "react";
import cx from "classnames";

import ResourceState from "../../ResourceState";
import TrendWidgetBody from "./TrendWidgetBody";
import { getDateAttributeForGranularity, getGranularitiesForDateDataSet } from "../../../utils/dateDataSets";
import useCatalogue from "../../../hooks/useCatalogue";
import { getAllMeasures, getMeasureByIdentifier } from "../../../utils/catalogue";

import sharedStyles from "../../../shared.module.scss";
import styles from "./TrendWidget.module.scss";

const TrendWidget = ({
    filters,
    maxMeasureCount = 10,
    headline,
    initialSelectedMeasuresIdentifiers = [],
    readOnly = false,
    ...restProps
}) => {
    const catalogue = useCatalogue();
    const { data } = catalogue;

    const defaultSelectedMeasures = useMemo(() => {
        return initialSelectedMeasuresIdentifiers.length
            ? initialSelectedMeasuresIdentifiers.map(measureId => {
                  return { measureId, isActive: true };
              })
            : [{ measureId: "", isActive: false }];
    }, [initialSelectedMeasuresIdentifiers]);
    const [selectedMeasures, setSelectedMeasures] = useState(defaultSelectedMeasures);
    const [selectedGranularity, setSelectedGranularity] = useState("GDC.time.quarter");

    // remove measures that are not available from the state
    useEffect(() => {
        if (!data) {
            return;
        }
        const validSelectedMeasures = selectedMeasures.filter(
            measure => !!getMeasureByIdentifier(data, measure.measureId),
        );
        if (validSelectedMeasures.length !== selectedMeasures.length) {
            console.warn(
                `TrendWidget: Removed measures (${selectedMeasures.length -
                    validSelectedMeasures.length} in total) because they are not available. Check your initialSelectedMeasuresIdentifiers settings.`,
            );
            setSelectedMeasures(
                selectedMeasures.filter(measure => !!getMeasureByIdentifier(data, measure.measureId)),
            );
        }
    }, [data]); // eslint-disable-line react-hooks/exhaustive-deps

    const updateSelectedMeasure = (index, measureUpdate) => {
        const newSelectedMeasures = selectedMeasures.slice(0, index);
        if (measureUpdate !== null) {
            newSelectedMeasures.push({ ...selectedMeasures[index], ...measureUpdate });
        }
        newSelectedMeasures.push(...selectedMeasures.slice(index + 1));
        setSelectedMeasures(newSelectedMeasures);
    };

    const availableGranularities =
        data && getGranularitiesForDateDataSet(data.dateDataSets[data.defaultDateDataSet]);

    const trendBy = data && getDateAttributeForGranularity(data, selectedGranularity);

    const lastMeasure = selectedMeasures[selectedMeasures.length - 1];

    const canAddMeasure =
        selectedMeasures.length < maxMeasureCount && (!lastMeasure || lastMeasure.measureId);

    return (
        <div className={cx(styles.TrendWidget, sharedStyles.Widget)} {...restProps}>
            <h3 className={cx(sharedStyles.Headline, styles.Headline)}>{headline}</h3>
            <ResourceState source={catalogue}>
                {data => (
                    <TrendWidgetBody
                        projectId={data.projectId}
                        filters={filters}
                        measures={getAllMeasures(data)}
                        selectedMeasures={selectedMeasures}
                        setSelectedMeasure={(index, measureId) =>
                            updateSelectedMeasure(index, measureId ? { measureId, isActive: true } : null)
                        }
                        setIsMeasureActive={(index, isActive) => updateSelectedMeasure(index, { isActive })}
                        addMeasure={() =>
                            setSelectedMeasures([...selectedMeasures, { measureId: "", isActive: false }])
                        }
                        canAddMeasure={canAddMeasure}
                        granularities={availableGranularities}
                        selectedGranularity={selectedGranularity}
                        setSelectedGranularity={setSelectedGranularity}
                        trendBy={trendBy}
                        readOnly={readOnly}
                    />
                )}
            </ResourceState>
        </div>
    );
};

export default TrendWidget;
