import React, { useState, useCallback, useMemo } from "react";
import { Container, Row, Col } from "react-grid-system";
import { Model } from "@gooddata/react-components";
import ReactModal from "react-modal";

import Page from "../components/Page";
import VisualizationBuilderWidget from "../components/visualizations/VisualizationBuilderWidget/VisualizationBuilderWidget";
import Drill from "../components/Drill";
import Button from "../components/controls/Button";
import { columnChartVisType } from "../components/visualizations/VisualizationBuilderWidget/visTypes";
import useCatalogue from "../hooks/useCatalogue";
import { itemize, collectAttributeDisplayForms, getAllAttributes } from "../utils/catalogue";
import Select from "../components/controls/Select";
import { getAttributeQualifier } from "../utils/bucketItems";

import sharedStyles from "../shared.module.scss";

const visTypePreset = columnChartVisType;

ReactModal.setAppElement("body");

//  This component edits a specified bucket and replaces it's last attribute,
// then filters the visualization by the same attribute value found in drillContext intersection
const VisualizationWithDrill = ({
    drillContext,
    visType,
    buckets,
    // an index of a bucket that should be replaced (viewBy; index 1)
    drillBucketIndex = 1,
    // what will be the attribute replaced by
    replacementAttributeIdentifier,
}) => {
    if (!drillContext || !visType || !buckets) {
        return null;
    }

    const originalBucket = buckets[drillBucketIndex];
    const originalBucketItems = originalBucket.items;
    const originalAttribute = originalBucketItems[originalBucketItems.length - 1];
    const originalAttributeIdentifier = originalAttribute.visualizationAttribute.displayForm.identifier;

    const modalVisType = {
        ...visType,
        buckets: [...buckets],
    };

    // replace items in a drillBucket
    modalVisType.buckets[drillBucketIndex] = {
        ...originalBucket,
        items: [...originalBucketItems],
    };
    const newBucketItems = modalVisType.buckets[drillBucketIndex].items;

    // replace last attribute with replacementAttribute
    newBucketItems[newBucketItems.length - 1] = Model.attribute(replacementAttributeIdentifier);

    // Find out attribute value of the drill intersection that matches original attribute
    const drillAttributeIntersectionPoint = drillContext.intersection.find(
        point => point.identifier === originalAttributeIdentifier,
    );

    // Filter visualization so that it shows only data that matches attribute value
    const drillFilter = Model.attributeFilter(originalAttributeIdentifier).in(
        drillAttributeIntersectionPoint.title,
    );

    return (
        <VisualizationBuilderWidget
            className={sharedStyles.ModalContent}
            headline={drillAttributeIntersectionPoint.title}
            visTypes={null}
            initialVisType={modalVisType}
            visualizationProps={{
                filters: [drillFilter],
            }}
        />
    );
};

const DrillToModal = ({
    initialDrillBy = null,
}) => {
    const [isModalOpen, setIsModalOpen] = useState(false);
    const catalogueState = useCatalogue();
    const [buckets, setBuckets] = useState(visTypePreset.buckets);
    const [drillBy, setDrillBy] = useState(initialDrillBy);
    const [drillContext, setDrillContext] = useState(null);
    const onDrill = useCallback(newDrillContext => {
        setDrillContext(newDrillContext);
        setIsModalOpen(true);
    }, []);

    const viewByBucket = buckets.find(bucket => {
        return bucket.prop === "viewBy";
    });
    const drillThroughIdentifier = viewByBucket && viewByBucket.items[0] ? getAttributeQualifier(viewByBucket.items[0]).identifier : null;

    const on = useMemo(() => {
        return {
            [drillThroughIdentifier]: onDrill,
        };
    }, [onDrill, drillThroughIdentifier]);

    const attributeDFs = catalogueState.data ? Object.values(collectAttributeDisplayForms(getAllAttributes(catalogueState.data))) : [];
    const catalogueItems = catalogueState.data ? attributeDFs.map(itemize) : [];

    const selectedDrillByItem = catalogueItems.find(catalogueItem =>
        catalogueItem.value && catalogueItem.value.meta.identifier === drillBy,
    );

    const drillBySelect = (
        <Select
            style={{display: 'inline-block', fontSize: 14, width: 200, textTransform: 'none'}}
            items={catalogueItems}
            selectedItem={selectedDrillByItem}
            onChange={(item) => setDrillBy(item ? item.value.meta.identifier : null)}
        />
    );

    return (
        <Page>
            <div className={sharedStyles.Header}>
                <Container fluid xs sm md lg>
                    <Row>
                        <Col className={sharedStyles.Column}>
                            <h2 className={sharedStyles.Title}>DrillToModal</h2>
                        </Col>
                    </Row>
                </Container>
            </div>
            <Container fluid xs sm md lg>
                <Row>
                    <Col>
                        <Drill on={on}>
                            {drillingProps => {
                                return (
                                    <>
                                        <VisualizationBuilderWidget
                                            headline={<>Drill to Modal by {drillBySelect}</>}
                                            visTypes={null}
                                            initialVisType={visTypePreset}
                                            visualizationProps={drillingProps}
                                            onBucketsChange={setBuckets}
                                        />
                                        <ReactModal
                                            isOpen={isModalOpen}
                                            onRequestClose={() => {
                                                setIsModalOpen(false);
                                            }}
                                            className={sharedStyles.Modal}
                                            overlayClassName={sharedStyles.ModalOverlay}
                                        >
                                            <Button
                                                onClick={() => setIsModalOpen(false)}
                                                className={sharedStyles.ModalClose}
                                            >
                                                ✕
                                            </Button>
                                            <VisualizationWithDrill
                                                visType={visTypePreset}
                                                buckets={buckets}
                                                replacementAttributeIdentifier={drillBy}
                                                drillContext={drillContext}
                                            />
                                        </ReactModal>
                                    </>
                                );
                            }}
                        </Drill>
                    </Col>
                </Row>
            </Container>
        </Page>
    );
};

export default DrillToModal;
