import sdk from "@gooddata/gooddata-js";
import flatMap from "lodash/flatMap";

import { MEASURE, DATE, ATTRIBUTE, ATTRIBUTE_AND_DATE } from "./bucketItems";

const catalogUri = projectId => `/gdc/internal/projects/${projectId}/loadCatalog`;

export const getCatalogueItems = (projectId, bucketItems) =>
    sdk.xhr.post(catalogUri(projectId), {
        data: {
            catalogRequest: {
                types: ["metric", "attribute", "fact"],
                paging: {
                    offset: 0,
                    limit: 100,
                },
                bucketItems,
                requiredDataSets: {
                    type: "PRODUCTION",
                },
            },
        },
    });

const catalogueTypeTargetCollections = {
    metric: "measures",
    attribute: "attributes",
    fact: "facts",
};
const validCatalogueItemTypes = Object.keys(catalogueTypeTargetCollections);

const isItemTypeValid = type => validCatalogueItemTypes.includes(type);
const isItemHidden = item => item.meta && item.meta.tags && item.meta.tags.split(" ").includes("hidden");
const getItemIdentifier = item => item.meta && item.meta.identifier;

const catalogueItemValidations = [
    {
        log: (item, type) => console.log("invalid item type", type),
        predicate: (item, type) => isItemTypeValid(type),
    },
    {
        log: (item, type) => console.log("skipping hidden item", item),
        predicate: (item, type) => !isItemHidden(item),
    },
    {
        log: (item, type) => console.log("missing identifier", item),
        predicate: (item, type) => !!getItemIdentifier(item),
    },
];

export const normalizeCatalogueResponse = catalogueResponse => {
    const initialResult = Object.values(catalogueTypeTargetCollections).reduce((acc, collection) => {
        acc[collection] = {};
        return acc;
    }, {});

    return catalogueResponse.data.catalogResponse.catalog.reduce((acc, wrappedItem) => {
        const type = Object.keys(wrappedItem)[0];
        const item = wrappedItem[type];

        const validationFailure = catalogueItemValidations.find(v => !v.predicate(item, type));
        if (validationFailure) {
            validationFailure.log(item, type);
            return acc;
        }

        const targetCollection = catalogueTypeTargetCollections[type];
        const identifier = getItemIdentifier(item);

        acc[targetCollection][identifier] = item;

        return acc;
    }, initialResult);
};

export const collectAttributeDisplayForms = (attributes = {}) => {
    return Object.values(attributes).reduce((displayForms, attribute) => {
        displayForms.push(...attribute.content.displayForms);
        return displayForms;
    }, []);
};

export const getAllAttributes = catalogue => catalogue.attributes || {};
export const getAttributeByIdentifier = (catalogue, identifier) => getAllAttributes(catalogue)[identifier];

export const getAttributeDisplayFormByIdentifier = (catalogue, attributeDisplayFormIdentifier) => {
    for (const attributeIdentifier in catalogue.attributes) {
        const attribute = getAttributeByIdentifier(catalogue, attributeIdentifier);
        for (const attributeDisplayForm of attribute.content.displayForms) {
            if (attributeDisplayForm.meta.identifier === attributeDisplayFormIdentifier) {
                return attributeDisplayForm;
            }
        }
    }
    return null;
};

export const getAllDateDataSets = catalogue => catalogue.dateDataSets || {};
export const getDateDataSetByIdentifier = (catalogue, identifier) =>
    getAllDateDataSets(catalogue)[identifier];

export const getDateDataSetDisplayFormByIdentifier = (catalogue, dateDataSetDisplayFormIdentifier) => {
    for (const dateDatasetIdentifier in catalogue.dateDataSets) {
        const dateDataSet = getDateDataSetByIdentifier(catalogue, dateDatasetIdentifier);
        for (const dateAttribute of dateDataSet.availableDateAttributes) {
            const { displayForms } = dateAttribute;
            for (const displayForm of displayForms) {
                if (displayForm.meta.identifier === dateDataSetDisplayFormIdentifier) {
                    return displayForm;
                }
            }
        }
    }
    return null;
};

export const getDateDataSetAttributeByDisplayFormByIdentifier = (
    catalogue,
    dateDataSetDisplayFormIdentifier,
) => {
    for (const dateDatasetIdentifier in catalogue.dateDataSets) {
        const dateDataSet = getDateDataSetByIdentifier(catalogue, dateDatasetIdentifier);
        for (const dateAttribute of dateDataSet.availableDateAttributes) {
            const { displayForms } = dateAttribute;
            for (const displayForm of displayForms) {
                if (displayForm.meta.identifier === dateDataSetDisplayFormIdentifier) {
                    return dateAttribute;
                }
            }
        }
    }
    return null;
};

export const getAllMeasures = catalogue => catalogue.measures || {};
export const getMeasureByIdentifier = (catalogue, identifier) => getAllMeasures(catalogue)[identifier];

export const getAllFacts = catalogue => catalogue.facts || {};
export const getFactByIdentifier = (catalogue, identifier) => getAllFacts(catalogue)[identifier];

export const getMeasureOrFactByIdentifier = (catalogue, identifier) =>
    getMeasureByIdentifier(catalogue, identifier) || getFactByIdentifier(catalogue, identifier);

export const getAllMeasuresAndFacts = catalogue => ({
    ...getAllMeasures(catalogue),
    ...getAllFacts(catalogue),
});


export const collectDateDatasetDisplayForms = (dateDataSets = {}) =>
    flatMap(Object.values(dateDataSets), dateDataSet =>
        flatMap(dateDataSet.availableDateAttributes, attribute => attribute.displayForms),
    );

export const getLabel = value => value && value.meta && value.meta.title;
export const itemize = value => ({ value, label: getLabel(value) });

export const getCatalogItemsByType = (catalogue, type) => {
    if (type === MEASURE) {
        return Object.values(getAllMeasures(catalogue)).map(itemize);
    }
    if (type === DATE) {
        const dateDataSets = collectDateDatasetDisplayForms(getAllDateDataSets(catalogue));
        return dateDataSets.map(itemize);
    }
    if (type === ATTRIBUTE) {
        const attributeDisplayForms = collectAttributeDisplayForms(getAllAttributes(catalogue));
        return attributeDisplayForms.map(itemize);
    }
    if (type === ATTRIBUTE_AND_DATE) {
        const attributeDisplayForms = collectAttributeDisplayForms(getAllAttributes(catalogue));
        const dateDataSets = collectDateDatasetDisplayForms(getAllDateDataSets(catalogue));
        return [...attributeDisplayForms.map(itemize), ...dateDataSets.map(itemize)];
    }
    return [];
};
